@@ -122,14 +122,23 @@ l_get_arrangeGrobArgs.l_facet_wrap <- function(target) {
                     )
                 }
 
-                grobs[[(i - 1) * ncol + j]] <- do.call(gridExtra::arrangeGrob, args)
+                index <- (i - 1) * ncol + j
+
+                args$name <- if(index == 1) {
+                    "plot"
+                } else {
+                    paste0("plot", index - 1) # start from 0
+                }
+
+                grobs[[index]] <- do.call(gridExtra::arrangeGrob, args)
             }
         }
     }
 
     args <- list(
         grobs = grobs,
-        layout_matrix = locations$layout_matrix
+        layout_matrix = locations$layout_matrix,
+        name = "plots"
     )
 
     # pack xlabel, title
